package com.eitc.common.utils;

import com.eitc.common.core.redis.RedisCache;
import com.eitc.common.utils.spring.SpringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Random;

/**
 * PatientBaseUtil
 *
 * @author mtx
 * @date: 2024/06/04 16:54
 */
@Component
public class PatientBaseUtil {
    public static RedisCache redisCache = SpringUtils.getBean(RedisCache.class);

    public static final int REDIS_EXPIRE = 60 * 60 * 24 * 2;

    @Value("${system-abbreviation}")
    public String systemAbbreviation;

    public static String abbreviation;

    /**
     * 被@PostConstruct修饰的方法会在服务器加载Servlet的时候运行，并且只会被服务器执行一次。
     * PostConstruct在构造函数之后执行,init()方法之前执行
     */
    @PostConstruct
    private void setSystemAbbreviation() {
        abbreviation = this.systemAbbreviation;
    }

    public static String findMedicalRecordNum() {
        String str = DateUtils.dateNow();
        String key = "medicalRecordNum:" + str;
        long incr = redisCache.incr(key, 1L);
        if (incr < 1) {
            incr = redisCache.incr(key, 1L);
        }
        // 有效期一年
        redisCache.expire(key, REDIS_EXPIRE);
        String sn = String.format("%03d", incr);
        return abbreviation + str + sn;
    }

    /**
     * 根据身份证号计算年龄
     *
     * @param identificationCard 身份证号
     * @return 年龄
     * @author mtx
     * @date: 2024/6/4 16:57
     */
    public static int findAge(String identificationCard) throws ParseException {
        // 获取当前日期
        Calendar calendar = Calendar.getInstance();
        int currentYear = calendar.get(Calendar.YEAR);
        // 提取身份证号中的出生日期
        String birthDateStr = identificationCard.substring(6, 14);
        // 将出生日期字符串转换为Date类型
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");
        Date birthDate = dateFormat.parse(birthDateStr);
        // 计算年龄
        calendar.setTime(birthDate);
        int birthYear = calendar.get(Calendar.YEAR);
        return currentYear - birthYear;
    }

    /**
     * 获取6位短信验证码
     *
     * @return 6位验证码
     * @author mtx
     * @date: 2024/6/12 10:00
     */
    public static int getCaptchaCode() {
        return 100000 + new Random().nextInt(900000);
    }

    /**
     * 获取用户密钥
     *
     * @param userId 用户id
     * @return key
     */
    public static String getSecretKey(String userId) {
        String key = "erm:app:login_secret_key:" + userId;

        String secretKey = redisCache.getCacheObject(key);
        if (secretKey == null || "".equals(secretKey)) {
            secretKey = createSeckKey();
            redisCache.setCacheObject(key, secretKey);
        }
        return secretKey;
    }

    /**
     * 生成8位秘钥
     *
     * @return 8位秘钥
     */
    private static String createSeckKey() {
        //需要生成几位
        int n = 8;
        //最终生成的字符串
        StringBuilder str = new StringBuilder();
        char s = 'a';
        char t = 'A';
        for (int i = 0; i < n; i++) {
            char r = s;
            if (i % 2 == 0) {
                r = t;
            }
            str.append((char) (Math.random() * 26 + r));
        }
        return str.toString();
    }

    /**
     * 判断时间是否重叠
     * true重叠。false不重叠
     *
     * @param start1 第一个时间段开始时间
     * @param end1   第一个时间段结束时间
     * @param start2 第二个时间段开始时间
     * @param end2   第二个时间段结束时间
     * @return 是否重叠
     * @date: 2023/8/10
     **/
    public static boolean isOverlapLocalTime(Date start1, Date end1, Date start2, Date end2) {
        return start1.compareTo(end2) < 0 && end1.compareTo(start2) > 0;
    }

}
