package com.eitc.patient.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.eitc.common.core.domain.AjaxResult;
import com.eitc.common.core.redis.RedisCache;
import com.eitc.common.utils.JWTUtil;
import com.eitc.common.utils.PatientBaseUtil;
import com.eitc.common.utils.SecurityUtils;
import com.eitc.patient.constant.PatientConstant;
import com.eitc.patient.domain.AppUser;
import com.eitc.patient.domain.AppUserPatients;
import com.eitc.patient.domain.Patient;
import com.eitc.patient.enums.AppUserLoginTypeEnum;
import com.eitc.patient.enums.PatientCreateTypeEnum;
import com.eitc.patient.mapper.AppUserMapper;
import com.eitc.patient.service.IAppUserPatientsService;
import com.eitc.patient.service.IAppUserService;
import com.eitc.patient.service.IPatientService;
import com.eitc.patient.vo.AppUserPatientVO;
import com.eitc.patient.vo.PatientVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * AppUserServiceImpl
 *
 * @author mtx
 * @date: 2024/06/11 17:37
 */
@Service
public class AppUserServiceImpl extends ServiceImpl<AppUserMapper, AppUser> implements IAppUserService {

    @Resource
    private RedisCache redisCache;
    @Resource
    private IAppUserService appUserService;
    @Resource
    private IAppUserPatientsService appUserPatientsService;
    @Resource
    private AppUserMapper appUserMapper;
    @Resource
    private IPatientService patientService;

    /**
     * @param phoneNumber 手机号
     * @return 是否成功
     * @author mtx
     * @date: 2024/6/12 11:20
     */
    @Override
    public AjaxResult sendCaptchaCode(String phoneNumber) {
        String key = PatientConstant.ERM_APP_CAPTCHA_CODE + phoneNumber;
        String repeat = key + ":repeat";
        int repeatNum = 1;
        if (redisCache.hasKey(repeat)) {
            repeatNum = redisCache.getCacheObject(repeat);
            int num = 5;
            if (repeatNum >= num) {
                return AjaxResult.error("验证码发送次数超过" + num + "次，请稍后再试");
            }
            repeatNum++;
        }
        redisCache.setCacheObject(repeat, repeatNum, 1, TimeUnit.MINUTES);

        int captchaCode = PatientBaseUtil.getCaptchaCode();
        redisCache.setCacheObject(key, captchaCode, 5, TimeUnit.MINUTES);
        return AjaxResult.success(captchaCode);
    }

    /**
     * 登录
     *
     * @param map 参数
     * @return 成功
     */
    @Override
    public AjaxResult login(Map<String, String> map) {
        AppUser appUser;
        String type = map.get("type");
        String phoneNumber = map.get("phoneNumber");
        if (String.valueOf(AppUserLoginTypeEnum.ACCOUNT.getCode()).equals(type)) {
            // 账号密码登录
            String password = map.get("password");
            if(StringUtils.isBlank(password)){
                return AjaxResult.error("密码不能为空");
            }
            AppUser user = getAppUserByPhoneNumber(phoneNumber);
            if (Objects.isNull(user)) {
                return AjaxResult.error("用户不存在或密码错误");
            }
            appUser = user;
            boolean flag = SecurityUtils.matchesPassword(password, appUser.getPassword());
            if (!flag) {
                return AjaxResult.error("用户不存在或密码错误");
            }
        } else if (String.valueOf(AppUserLoginTypeEnum.CAPTCHA.getCode()).equals(type)) {
            // 手机验证码登录
            String captchaCode = map.get("captchaCode");
            if (checkCaptchaCode(phoneNumber, captchaCode)) {
                return AjaxResult.error("验证码错误");
            }
            AppUser user = getAppUserByPhoneNumber(phoneNumber);
            if (Objects.isNull(user)) {
                return AjaxResult.error("用户不存在");
            }
            appUser = user;
        } else {
            return AjaxResult.error("登录方式错误");
        }

        Map<String, String> payload = getTokenByAppUserInfo(appUser);

        return AjaxResult.success(payload);
    }

    private Map<String, String> getTokenByAppUserInfo(AppUser appUser) {
        Map<String, String> payload = new HashMap<>();
        payload.put("id", appUser.getId());
        payload.put("phoneNumber", appUser.getPhoneNumber());
        payload.put("identificationCard", appUser.getIdentificationCard());

        String secretKey = PatientBaseUtil.getSecretKey(appUser.getId());
        //生成JWt的令牌
        String token = JWTUtil.getToken(payload, secretKey);
        payload.put("token", token);
        // 删除验证码
        delCaptchaCode(appUser.getPhoneNumber());
        return payload;
    }

    /**
     * 设置密码
     *
     * @param map 参数
     * @return 返回结果
     * @author mtx
     * @date: 2024/6/13 9:26
     */
    @Override
    public AjaxResult setPassword(Map<String, String> map) {
        String phoneNumber = map.get("phoneNumber");
        String captchaCode = map.get("captchaCode");
        String password = map.get("password");
        String confirmPassword = map.get("confirmPassword");
        if (StringUtils.isEmpty(phoneNumber)) {
            return AjaxResult.error("手机号不能为空");
        }
        if (StringUtils.isEmpty(captchaCode)) {
            return AjaxResult.error("验证码不能为空");
        }
        if (StringUtils.isEmpty(password)) {
            return AjaxResult.error("密码不能为空");
        }

        AppUser appUser = getAppUserByPhoneNumber(phoneNumber);
        if (Objects.isNull(appUser)) {
            return AjaxResult.error("用户不存在");
        }
        if (checkCaptchaCode(phoneNumber, captchaCode)) {
            return AjaxResult.error("验证码错误");
        }
        if (!password.equals(confirmPassword)) {
            return AjaxResult.error("两次密码不一致");
        }

        appUserService.update(Wrappers.<AppUser>lambdaUpdate()
                .eq(AppUser::getId, appUser.getId())
                .set(AppUser::getPassword, SecurityUtils.encryptPassword(password)));

        // 删除验证码
        delCaptchaCode(phoneNumber);
        return AjaxResult.success();
    }

    /**
     * 修改手机号
     *
     * @param map 参数
     * @return 返回结果
     * @author mtx
     * @date: 2024/6/13 9:29
     */
    @Override
    public AjaxResult updatePhoneNumber(Map<String, String> map) {
        String appPhoneNumber = JWTUtil.getAppPhoneNumber();
        String phoneNumber = map.get("phoneNumber");
        String captchaCode = map.get("captchaCode");

        if (appPhoneNumber.equals(phoneNumber)) {
            return AjaxResult.error("手机号一样");
        }
        if (StringUtils.isEmpty(phoneNumber)) {
            return AjaxResult.error("手机号不能为空");
        }
        if (StringUtils.isEmpty(captchaCode)) {
            return AjaxResult.error("验证码不能为空");
        }
        if (checkCaptchaCode(phoneNumber, captchaCode)) {
            return AjaxResult.error("验证码错误");
        }

        appUserService.update(Wrappers.<AppUser>lambdaUpdate()
                .eq(AppUser::getId, JWTUtil.getAppUserId())
                .set(AppUser::getPhoneNumber, phoneNumber));

        // 删除验证码
        delCaptchaCode(phoneNumber);
        return AjaxResult.success();
    }

    /**
     * 锁定用户&解锁用户
     *
     * @param map id type 类型（1锁定用户，0解锁用户）
     * @return 返回结果
     * @author mtx
     * @date: 2024/6/13 10:27
     */
    @Override
    public AjaxResult locking(Map<String, String> map) {
        String id = map.get("id");
        String type = map.get("type");
        if (StringUtils.isEmpty(id)) {
            return AjaxResult.error("id不能为空");
        }
        if (StringUtils.isEmpty(type)) {
            return AjaxResult.error("type不能为空");
        }
        if ("0".equals(type)) {
            // 解锁
            this.revert(id);
        } else if ("1".equals(type)) {
            // 锁定用户
            this.removeById(id);
        } else {
            return AjaxResult.error("type错误");
        }

        return AjaxResult.success();
    }

    /**
     * 回复删除数据
     *
     * @param id id
     * @author mtx
     * @date: 2024/6/13 10:33
     */
    @Override
    public void revert(String id) {
        appUserMapper.updateRevert(id);
    }

    /**
     * 添加就诊人
     *
     * @param vo 参数
     * @author mtx
     * @date: 2024/6/14 11:23
     */
    @Override
    public void add(AppUserPatientVO vo) {
        PatientVO patientVO = new PatientVO();
        String userId = JWTUtil.getAppUserId();
        // 添加患者信息
        Patient one = patientService.getOne(Wrappers.<Patient>lambdaQuery().eq(Patient::getIdentificationCard, vo.getIdentificationCard()));
        if (Objects.nonNull(one)) {
            patientVO.setId(one.getId());
        }
        Patient patient = savePatient(vo, patientVO);

        // 如果是默认，则将其他的数据改为非默认
        AppUserPatients appUserPatients = new AppUserPatients();
        if (Objects.nonNull(vo.getIsDefault()) && 1 == vo.getIsDefault()) {
            appUserPatientsService.update(Wrappers.<AppUserPatients>lambdaUpdate()
                    .set(AppUserPatients::getIsDefault, 0)
                    .eq(AppUserPatients::getAppUserId, userId));
            appUserPatients.setIsDefault(vo.getIsDefault());
        } else {
            appUserPatients.setIsDefault(0);
        }

        // 关联账号
        appUserPatients.setAppUserId(userId);
        appUserPatients.setPatientId(patient.getId());
        appUserPatients.setRelationship(vo.getUserRelationship());
        appUserPatientsService.remove(Wrappers.<AppUserPatients>lambdaQuery()
                .eq(AppUserPatients::getAppUserId, userId)
                .eq(AppUserPatients::getPatientId, patient.getId()));
        appUserPatientsService.save(appUserPatients);
    }

    @Override
    public void edit(AppUserPatientVO vo) {
        PatientVO patientVO = new PatientVO();
        String userId = JWTUtil.getAppUserId();
        // 添加患者信息
        Patient one = patientService.getById(vo.getPatientId());
        if (Objects.nonNull(one)) {
            patientVO.setId(one.getId());
        } else {
            throw new IllegalArgumentException("患者信息不存在");
        }
        Patient patient = savePatient(vo, patientVO);

        // 如果是默认，则将其他的数据改为非默认
        Integer isDefault = 0;
        if (Objects.nonNull(vo.getIsDefault()) && 1 == vo.getIsDefault()) {
            appUserPatientsService.update(Wrappers.<AppUserPatients>lambdaUpdate()
                    .set(AppUserPatients::getIsDefault, 0)
                    .eq(AppUserPatients::getAppUserId, userId));
            isDefault = vo.getIsDefault();
        }

        // 关联账号
        appUserPatientsService.update(Wrappers.<AppUserPatients>lambdaUpdate()
                .set(AppUserPatients::getIsDefault, isDefault)
                .set(AppUserPatients::getRelationship, vo.getUserRelationship())
                .eq(AppUserPatients::getAppUserId, userId)
                .eq(AppUserPatients::getPatientId, patient.getId()));
    }

    private Patient savePatient(AppUserPatientVO vo, PatientVO patientVO) {
        patientVO.setFamilyHistory(vo.getFamilyHistory());
        patientVO.setAllergyHistory(vo.getAllergyHistory());
        patientVO.setPastHistory(vo.getPastHistory());
        patientVO.setPatientName(vo.getPatientName());
        patientVO.setDocumentType(vo.getDocumentType());
        patientVO.setIdentificationCard(vo.getIdentificationCard());
        patientVO.setPatientPhone(vo.getPatientPhone());
        patientVO.setHomeAddress(vo.getHomeAddress());
        patientVO.setCreateType(PatientCreateTypeEnum.APP.getCode());
        return patientService.addOrEdit(patientVO);
    }

    /**
     * 通过手机号获取app用户信息
     *
     * @param phoneNumber 手机号
     * @return 返回结果
     * @author mtx
     * @date: 2024/6/13 9:59
     */
    public AppUser getAppUserByPhoneNumber(String phoneNumber) {
        return appUserService.getOne(Wrappers.<AppUser>lambdaQuery().eq(AppUser::getPhoneNumber, phoneNumber));
    }

    /**
     * 注册
     *
     * @param map 参数
     * @return 用户
     */
    @Override
    public boolean register(Map<String, String> map) {
        String phoneNumber = map.get("phoneNumber");
        if (checkCaptchaCode(phoneNumber, map.get("captchaCode"))) {
            throw new IllegalArgumentException("验证码错误");
        }
        if (Objects.nonNull(getAppUserByPhoneNumber(phoneNumber))) {
            throw new IllegalArgumentException("手机号已注册");
        }

        // 删除验证码
        delCaptchaCode(phoneNumber);

        AppUser appUser = new AppUser();
        appUser.setPhoneNumber(phoneNumber);
        appUser.setIdentificationCard(map.get("identificationCard"));
        String password = map.get("password");
        if (StringUtils.isNotEmpty(password)) {
            appUser.setPassword(SecurityUtils.encryptPassword(password));
        }
        appUser.setCreateUser(phoneNumber);
        appUser.setUpdateUser(phoneNumber);
        return appUserService.save(appUser);
    }

    /**
     * APP切换诊所时同步数据
     *
     * @param beforeAppUser 前一次登录用户信息
     * @return 返回结果
     * @author mtx
     * @date: 2024/6/20 8:53
     */
    @Override
    @Transactional(rollbackFor = {Exception.class})
    public AjaxResult synchronizeData(AppUser beforeAppUser) {
        String phoneNumber = beforeAppUser.getPhoneNumber();
        String identificationCard = beforeAppUser.getIdentificationCard();
        if(StringUtils.isEmpty(phoneNumber)){
            return AjaxResult.error("手机号不能为空");
        }
        if(StringUtils.isEmpty(identificationCard)){
            return AjaxResult.error("身份证号不能为空");
        }
        String password = beforeAppUser.getPassword();
        // 保存用户信息
        AppUser saveAppUser = new AppUser();
        AppUser appUser = getAppUserByPhoneNumber(phoneNumber);
        if (Objects.nonNull(appUser)) {
            saveAppUser.setId(appUser.getId());
        } else {
            saveAppUser.setPhoneNumber(phoneNumber);
        }
        saveAppUser.setIdentificationCard(identificationCard);
        saveAppUser.setPassword(password);
        this.saveOrUpdate(saveAppUser);
        // 生成token
        Map<String, String> payload = getTokenByAppUserInfo(saveAppUser);
        logout();
        return AjaxResult.success(payload);
    }

    @Override
    public void logout() {
        try {
            String userId = JWTUtil.getAppUserId();
            String key = PatientConstant.ERM_APP_LOGIN_SECRET_KEY + userId;
            redisCache.deleteObject(key);
        } catch (Exception ignored) {
        }
    }

    /**
     * 验证短信验证码时候正确
     *
     * @param phoneNumber 手机号
     * @param captchaCode 录入的验证码
     * @return 是否相同 已经取反
     */
    public boolean checkCaptchaCode(String phoneNumber, String captchaCode) {
        String key = PatientConstant.ERM_APP_CAPTCHA_CODE + phoneNumber;
        if (redisCache.hasKey(key)) {
            int code = redisCache.getCacheObject(key);
            return !String.valueOf(code).equals(captchaCode);
        }
        return true;
    }

    /**
     * 删除验证码
     *
     * @param phoneNumber 手机号
     */
    public void delCaptchaCode(String phoneNumber) {
        String key = PatientConstant.ERM_APP_CAPTCHA_CODE + phoneNumber;
        redisCache.deleteObject(key);
    }
}
