package com.eitc.patient.service.impl;

import com.alibaba.fastjson2.JSONArray;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.eitc.patient.constant.PatientConstant;
import com.eitc.patient.domain.DiagnosisType;
import com.eitc.patient.domain.DiagnosisTypeTemplate;
import com.eitc.patient.domain.MedicalHistory;
import com.eitc.patient.domain.MedicalRecord;
import com.eitc.patient.domain.MedicalRecordDetails;
import com.eitc.patient.domain.PatientAppointment;
import com.eitc.patient.enums.MedicalHistoryDiseaseSourceEnum;
import com.eitc.patient.enums.MedicalHistoryDiseaseTypeEnum;
import com.eitc.patient.enums.MedicalRecordStatusEnum;
import com.eitc.patient.enums.PatientAppointmentStatusEnum;
import com.eitc.patient.mapper.MedicalRecordMapper;
import com.eitc.patient.service.IDiagnosisTypeService;
import com.eitc.patient.service.IDiagnosisTypeTemplateService;
import com.eitc.patient.service.IMedicalHistoryService;
import com.eitc.patient.service.IMedicalRecordDetailsService;
import com.eitc.patient.service.IMedicalRecordService;
import com.eitc.patient.service.IPatientAppointmentService;
import com.eitc.patient.vo.FormattedPartsVO;
import com.eitc.patient.vo.MedicalRecordDetailsVO;
import com.eitc.patient.vo.MedicalRecordParamVO;
import com.eitc.patient.vo.MedicalRecordVO;
import com.eitc.system.utils.UserCacheUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * MedicalRecordServiceImpl
 *
 * @author mtx
 * @date: 2024/05/17 15:39
 */
@Service
public class MedicalRecordServiceImpl extends ServiceImpl<MedicalRecordMapper, MedicalRecord> implements IMedicalRecordService {

    @Resource
    private IMedicalHistoryService medicalHistoryService;
    @Resource
    private IMedicalRecordDetailsService medicalRecordDetailsService;
    @Resource
    private IDiagnosisTypeTemplateService diagnosisTypeTemplateService;
    @Resource
    private IDiagnosisTypeService diagnosisTypeService;
    @Resource
    private IPatientAppointmentService patientAppointmentService;

    /**
     * 保存病历
     *
     * @param vo 参数
     * @author mtx
     * @date: 2024/5/22 10:00
     */
    @Override
    @Transactional(rollbackFor = {Exception.class})
    public void addOrEdit(MedicalRecordParamVO vo) {
        // 保存病历信息
        MedicalRecord medicalRecord = new MedicalRecord();
        medicalRecord.setAttendingDoctorId(vo.getAttendingDoctorId());
        medicalRecord.setAttendingDoctorName(UserCacheUtil.getSysUserById(vo.getAttendingDoctorId()).getNickName());
        medicalRecord.setPatientId(vo.getPatientId());
        medicalRecord.setStatus(vo.getStatus());
        medicalRecord.setOutpatientService(UserCacheUtil.getClinicName());
        medicalRecord.setDepartment(UserCacheUtil.getSysDeptByUserId(vo.getAttendingDoctorId()).getDeptName());
        medicalRecord.setCost(BigDecimal.ZERO);

        if (Objects.nonNull(vo.getId())) {
            medicalRecord.setId(vo.getId());
        } else {
            long count = this.count(Wrappers.<MedicalRecord>lambdaQuery().eq(MedicalRecord::getPatientId, medicalRecord.getPatientId()));
            medicalRecord.setVisitType(count > 0 ? 2 : 1);
        }
        this.saveOrUpdate(medicalRecord);
        // 保存个人录入的病史
        medicalHistoryService.saveMedicalHistoryToMedicalRecordIdByPatientId(medicalRecord.getPatientId(), medicalRecord.getId());

        // 修改预约状态
        if (StringUtils.isNotBlank(vo.getAppointmentId())) {
            PatientAppointment appointment = new PatientAppointment();
            appointment.setId(vo.getAppointmentId());
            appointment.setMedicalRecordId(medicalRecord.getId());
            appointment.setAppointmentStatus(MedicalRecordStatusEnum.ALREADY_COMPLETE.getCode().equals(medicalRecord.getStatus()) ?
                    PatientAppointmentStatusEnum.TREATMENT_COMPLETED.getCode() : PatientAppointmentStatusEnum.IN_TREATMENT.getCode());
            patientAppointmentService.updateById(appointment);
        }

        List<MedicalRecordVO> dataList = vo.getDataList();
        if (CollectionUtils.isEmpty(dataList)) {
            throw new RuntimeException("病历详情不能为空");
        }

        // 保存病历及详情
        AtomicInteger num = new AtomicInteger(1);
        List<MedicalRecordDetails> recordDetailsList = new ArrayList<>();
        dataList.forEach(medicalRecordVO -> {
            List<MedicalRecordDetailsVO> data = medicalRecordVO.getData();

            data.forEach(medicalRecordDetailsVO -> {
                String templateId = medicalRecordDetailsVO.getDiagnosisTypeTemplateId();
                DiagnosisTypeTemplate diagnosisTypeTemplate = diagnosisTypeTemplateService.getById(templateId);
                if (Objects.isNull(diagnosisTypeTemplate)) {
                    throw new RuntimeException("诊断分类模板表获取失败");
                }
                String option = medicalRecordDetailsVO.getFormattedParts().stream()
                        .filter(li -> "select".equals(li.getType()))
                        .map(FormattedPartsVO::getValue)
                        .collect(Collectors.joining(","));

                MedicalRecordDetails recordDetails = new MedicalRecordDetails();
                recordDetails.setMedicalRecordId(medicalRecord.getId());
                recordDetails.setDiagnosisTypeTemplateId(templateId);
                recordDetails.setDiagnosisTypeId(diagnosisTypeTemplate.getDiagnosisTypeId());
                DiagnosisType diagnosisType = diagnosisTypeService.getById(diagnosisTypeTemplate.getDiagnosisTypeId());
                if (Objects.nonNull(diagnosisType)) {
                    recordDetails.setDiagnosisTypeName(diagnosisType.getDiagnosisName());
                }
                recordDetails.setToothPosition(medicalRecordVO.getToothPosition());
                recordDetails.setTemplateName(diagnosisTypeTemplate.getTemplateName());
                recordDetails.setTemplateExample(diagnosisTypeTemplate.getTemplateExample());
                recordDetails.setTemplateInfo(diagnosisTypeTemplate.getTemplateInfo());
                recordDetails.setTemplateOption(option);
                recordDetails.setOrderNum(num.getAndIncrement());
                recordDetails.setContentInfo(
                        encapsulationTemplateInfo(recordDetails.getTemplateInfo(), recordDetails.getTemplateOption()));

                List<String> list = new ArrayList<>();
                String patientMedicalRecord = medicalRecordDetailsVO.getPatientMedicalRecord();
                if (StringUtils.isNotBlank(patientMedicalRecord)) {
                    list.addAll(Arrays.asList(patientMedicalRecord.split(",")));
                }
                if ("过敏史".equals(medicalRecordDetailsVO.getTemplateName())) {
                    String collect = medicalRecordDetailsVO.getFormattedPartsList().stream()
                            .filter(li -> !list.contains(li)).collect(Collectors.joining(","));

                    // 保存医生填写的病史
                    medicalHistoryService.saveMedicalHistoryList(
                            collect, null, null,
                            medicalRecord.getId(), null, MedicalHistoryDiseaseSourceEnum.DOCTOR.getCode());
                }
                if ("既往史".equals(medicalRecordDetailsVO.getTemplateName())) {
                    String collect = medicalRecordDetailsVO.getFormattedPartsList().stream()
                            .filter(li -> !list.contains(li)).collect(Collectors.joining(","));
                    // 保存医生填写的病史
                    medicalHistoryService.saveMedicalHistoryList(
                            null, collect, null,
                            medicalRecord.getId(), null, MedicalHistoryDiseaseSourceEnum.DOCTOR.getCode());
                }
                if ("家族史".equals(medicalRecordDetailsVO.getTemplateName())) {
                    String collect = medicalRecordDetailsVO.getFormattedPartsList().stream()
                            .filter(li -> !list.contains(li)).collect(Collectors.joining(","));
                    // 保存医生填写的病史
                    medicalHistoryService.saveMedicalHistoryList(
                            null, null, collect,
                            medicalRecord.getId(), null, MedicalHistoryDiseaseSourceEnum.DOCTOR.getCode());
                }

                String jsonString = JSONArray.toJSONString(medicalRecordDetailsVO.getFormattedParts());
                recordDetails.setTemplateView(jsonString);

                recordDetailsList.add(recordDetails);
            });
        });

        // 删除后新增
        medicalRecordDetailsService.remove(Wrappers.<MedicalRecordDetails>lambdaQuery().eq(MedicalRecordDetails::getMedicalRecordId, medicalRecord.getId()));
        medicalRecordDetailsService.saveBatch(recordDetailsList);
    }

    /**
     * 封装病历信息
     *
     * @param medicalRecord 病历信息
     * @return 病历信息
     * @author mtx
     * @date: 2024/5/23 9:17
     */
    @Override
    public MedicalRecordParamVO getMedicalRecordVO(MedicalRecord medicalRecord) {
        MedicalRecordParamVO vo = new MedicalRecordParamVO();
        vo.setId(medicalRecord.getId());
        vo.setVisitType(medicalRecord.getVisitType());
        vo.setStatus(medicalRecord.getStatus());
        vo.setPatientId(medicalRecord.getPatientId());
        vo.setAttendingDoctorId(medicalRecord.getAttendingDoctorId());
        vo.setAttendingDoctorName(medicalRecord.getAttendingDoctorName());

        vo.setDepartment(medicalRecord.getDepartment());
        vo.setCost(BigDecimal.ZERO);
        vo.setClinicName(medicalRecord.getOutpatientService());
        vo.setCreateDate(medicalRecord.getCreateDate());

        List<MedicalRecordVO> dataList = new ArrayList<>();
        List<MedicalRecordDetails> list = medicalRecordDetailsService.list(Wrappers.<MedicalRecordDetails>lambdaQuery()
                .eq(MedicalRecordDetails::getMedicalRecordId, medicalRecord.getId())
                .orderByAsc(MedicalRecordDetails::getOrderNum));

        Map<String, List<MedicalRecordDetails>> collect = list.stream().collect(Collectors.groupingBy(MedicalRecordDetails::getToothPosition));
        for (String key : collect.keySet()) {
            MedicalRecordVO medicalRecordVO = new MedicalRecordVO();
            medicalRecordVO.setToothPosition(key);

            List<MedicalRecordDetails> detailsList = collect.get(key);
            if (CollectionUtils.isEmpty(detailsList)) {
                medicalRecordVO.setData(new ArrayList<>());
            } else {
                List<MedicalRecordDetailsVO> details = new ArrayList<>();
                detailsList.stream().sorted(Comparator.comparing(MedicalRecordDetails::getOrderNum)).forEach(li -> {
                    MedicalRecordDetailsVO medicalRecordDetailsVO = new MedicalRecordDetailsVO();

                    if ("诊断".equals(li.getTemplateName()) && StringUtils.isBlank(vo.getDiagnosisTypeCn())) {
                        vo.setDiagnosisTypeCn(li.getDiagnosisTypeName());
                    }
                    if ("治疗计划".equals(li.getTemplateName()) && StringUtils.isBlank(vo.getTreatment())) {
                        vo.setTreatment(li.getContentInfo());
                    }

                    medicalRecordDetailsVO.setMedicalRecordDetailsId(li.getId());
                    medicalRecordDetailsVO.setDiagnosisTypeTemplateId(li.getDiagnosisTypeTemplateId());
                    medicalRecordDetailsVO.setDiagnosisTypeId(li.getDiagnosisTypeId());
                    medicalRecordDetailsVO.setDiagnosisTypeCn(li.getDiagnosisTypeName());
                    medicalRecordDetailsVO.setTemplateName(li.getTemplateName());
                    medicalRecordDetailsVO.setTemplateExample(li.getTemplateExample());
                    medicalRecordDetailsVO.setContentInfo(li.getContentInfo());
                    medicalRecordDetailsVO.setTemplateInfo(li.getTemplateInfo());
                    medicalRecordDetailsVO.setTemplateOption(li.getTemplateOption());

                    List<MedicalHistory> histories = medicalHistoryService.list(Wrappers.<MedicalHistory>lambdaQuery().eq(MedicalHistory::getMedicalRecordId, li.getMedicalRecordId()));
                    if (MedicalHistoryDiseaseTypeEnum.ALLERGY_HISTORY.getInfo().equals(li.getTemplateName())) {
                        findMedicalHistory(medicalRecordDetailsVO, histories, MedicalHistoryDiseaseTypeEnum.ALLERGY_HISTORY.getCode());
                    } else if (MedicalHistoryDiseaseTypeEnum.PAST_HISTORY.getInfo().equals(li.getTemplateName())) {
                        findMedicalHistory(medicalRecordDetailsVO, histories, MedicalHistoryDiseaseTypeEnum.PAST_HISTORY.getCode());
                    } else if (MedicalHistoryDiseaseTypeEnum.FAMILY_HISTORY.getInfo().equals(li.getTemplateName())) {
                        findMedicalHistory(medicalRecordDetailsVO, histories, MedicalHistoryDiseaseTypeEnum.FAMILY_HISTORY.getCode());
                    }

                    medicalRecordDetailsVO.setFormattedParts(JSONArray.parseArray(li.getTemplateView(), FormattedPartsVO.class));
                    details.add(medicalRecordDetailsVO);
                });

                medicalRecordVO.setData(details);
            }
            dataList.add(medicalRecordVO);
        }

        vo.setDataList(dataList);
        return vo;
    }

    private void findMedicalHistory(MedicalRecordDetailsVO medicalRecordDetailsVO, List<MedicalHistory> histories, Integer diseaseType) {
        List<MedicalHistory> collect = histories.stream().filter(l -> diseaseType.equals(l.getDiseaseType())).collect(Collectors.toList());
        medicalRecordDetailsVO.setPatientMedicalRecord(collect.stream()
                .filter(l -> MedicalHistoryDiseaseSourceEnum.PATIENT.getCode().equals(l.getDiseaseSource()))
                .map(MedicalHistory::getDiseaseContent).collect(Collectors.joining(",")));
        medicalRecordDetailsVO.setDoctorMedicalRecord(collect.stream()
                .filter(l -> MedicalHistoryDiseaseSourceEnum.DOCTOR.getCode().equals(l.getDiseaseSource()))
                .map(MedicalHistory::getDiseaseContent).collect(Collectors.joining(",")));

        medicalRecordDetailsVO.setFormattedPartsList(collect.stream().map(MedicalHistory::getDiseaseContent).collect(Collectors.toList()));
    }

    /**
     * 根据模板选项，将模板选项替换成模板示例
     *
     * @param templateInfo   模板信息
     * @param templateOption 模板选项
     * @return 替换后的数据
     * @author mtx
     * @date: 2024/5/23 9:15
     */
    public String encapsulationTemplateInfo(String templateInfo, String templateOption) {
        String[] split = templateOption.split(",");
        String keyPrefix = PatientConstant.DIAGNOSIS_TYPE_TEMPLATE_KEY_PREFIX;
        int count = 0, i = 0;
        while (templateInfo.indexOf(keyPrefix, i) >= 0) {
            count++;
            i = templateInfo.indexOf(keyPrefix, i) + keyPrefix.length();
        }

        for (int j = 0; j < count; j++) {
            int i1 = j + 1;
            String s = "";
            try {
                s = split[j];
            } catch (Exception ignored) {
            }
            templateInfo = templateInfo.replace(keyPrefix + i1, s);
        }
        return templateInfo;
    }
}
