package com.eitc.patient.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.eitc.common.annotation.Log;
import com.eitc.common.core.controller.BaseController;
import com.eitc.common.core.domain.AjaxResult;
import com.eitc.common.enums.BusinessType;
import com.eitc.common.utils.bean.BeanUtils;
import com.eitc.patient.domain.MedicalHistory;
import com.eitc.patient.domain.Patient;
import com.eitc.patient.domain.PatientAppointment;
import com.eitc.patient.enums.MedicalHistoryDiseaseSourceEnum;
import com.eitc.patient.enums.MedicalHistoryDiseaseTypeEnum;
import com.eitc.patient.service.IMedicalHistoryService;
import com.eitc.patient.service.IPatientAppointmentService;
import com.eitc.patient.service.IPatientService;
import com.eitc.patient.vo.PatientVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * PatientController
 *
 * @author mtx
 * @date: 2024/05/17 15:52
 */
@Api(tags = "患者")
@RestController
@RequestMapping("/patient/patient")
public class PatientController extends BaseController {

    @Resource
    private IPatientService patientService;
    @Resource
    private IMedicalHistoryService medicalHistoryService;
    @Resource
    private IPatientAppointmentService patientAppointmentService;

    /**
     * 查询患者
     */
    @GetMapping("/list")
    @ApiOperation("查询患者")
    public AjaxResult list(Patient patient) {
        LambdaQueryWrapper<Patient> eq = Wrappers.<Patient>lambdaQuery()
                .like(StringUtils.isNotBlank(patient.getMedicalRecordNum()), Patient::getMedicalRecordNum, patient.getMedicalRecordNum())
                .like(StringUtils.isNotBlank(patient.getPatientName()), Patient::getPatientName, patient.getPatientName())
                .like(StringUtils.isNotBlank(patient.getIdentificationCard()), Patient::getIdentificationCard, patient.getIdentificationCard())
                .orderByDesc(Patient::getCreateDate);
        List<Patient> list = patientService.list(eq);
        if (CollectionUtils.isEmpty(list)) {
            return success(new ArrayList<>());
        }
        List<PatientVO> listVO = new ArrayList<>();
        for (Patient li : list) {
            listVO.add(getPatientVO(li));
        }
        return success(listVO);
    }

    private PatientVO getPatientVO(Patient li) {
        PatientVO vo = new PatientVO();
        BeanUtils.copyProperties(li, vo);

        List<PatientAppointment> list = patientAppointmentService.list(Wrappers.<PatientAppointment>lambdaQuery()
                .eq(PatientAppointment::getAppointmentPersonId, li.getId())
                .orderByAsc(PatientAppointment::getCreateDate));
        if(CollectionUtils.isNotEmpty(list)){
            Optional<PatientAppointment> first = list.stream().findFirst();
            if(first.isPresent()){
                PatientAppointment one = first.get();
                vo.setVisitTime(one.getCreateDate());
                vo.setVisitingPhysician(one.getAttendingDoctor());
                vo.setAppointmentPerson(one.getRegisterUser());
            }
        }

        List<MedicalHistory> histories = medicalHistoryService.list(Wrappers.<MedicalHistory>lambdaQuery()
                .eq(MedicalHistory::getDiseaseSource, MedicalHistoryDiseaseSourceEnum.PATIENT.getCode())
                .eq(MedicalHistory::getPatientId, li.getId()));
        if (CollectionUtils.isNotEmpty(histories)) {
            vo.setAllergyHistory(
                    histories.stream()
                            .filter(entry -> MedicalHistoryDiseaseTypeEnum.ALLERGY_HISTORY.getCode().equals(entry.getDiseaseType()))
                            .map(MedicalHistory::getDiseaseContent)
                            .collect(Collectors.joining(","))
            );
            vo.setPastHistory(
                    histories.stream()
                            .filter(entry -> MedicalHistoryDiseaseTypeEnum.PAST_HISTORY.getCode().equals(entry.getDiseaseType()))
                            .map(MedicalHistory::getDiseaseContent)
                            .collect(Collectors.joining(","))
            );
            vo.setFamilyHistory(
                    histories.stream()
                            .filter(entry -> MedicalHistoryDiseaseTypeEnum.FAMILY_HISTORY.getCode().equals(entry.getDiseaseType()))
                            .map(MedicalHistory::getDiseaseContent)
                            .collect(Collectors.joining(","))
            );
        }
        return vo;
    }

    /**
     * 获取患者详细信息
     */
    @GetMapping(value = "/detail")
    @ApiOperation("获取患者详细信息")
    public AjaxResult detail(String id) {
        Patient patient = patientService.getById(id);
        return success(getPatientVO(patient));
    }

    /**
     * 新增患者
     */
    @Log(title = "新增患者", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation("新增患者")
    public AjaxResult add(@Valid @RequestBody PatientVO vo) {
        patientService.addOrEdit(vo);
        return success();
    }

    /**
     * 修改患者
     */
    @Log(title = "修改患者", businessType = BusinessType.UPDATE)
    @PutMapping
    @ApiOperation("修改患者")
    public AjaxResult edit(@Valid @RequestBody PatientVO vo) {
        if (StringUtils.isEmpty(vo.getId())) {
            return error();
        }
        patientService.addOrEdit(vo);
        return success();
    }

    /**
     * 删除患者
     */
    @Log(title = "删除患者", businessType = BusinessType.DELETE)
    @DeleteMapping("/remove")
    @ApiOperation("删除患者")
    public AjaxResult remove(String ids) {
        List<String> idList = Arrays.asList(ids.split(","));
        return toAjax(patientService.delete(idList));
    }

}
